local FOLDER_NAME, private = ...
local core = LibStub("AceAddon-3.0"):GetAddon(FOLDER_NAME)
local L = LibStub("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local strformat, pairs, tinsert, unpack, strsub, type  = string.format, pairs, tinsert, unpack, string.sub, type
local GetItemIcon = GetItemIcon
local LTT = LibStub:GetLibrary("LibQTip-1.0")
local tooltip, xp, xpr, xpg, xpb, xpa, tracktext, fname
local font1 = CreateFont("Header1")
font1:SetFont(GameTooltipHeaderText:GetFont(), 15)
local font2 = CreateFont("Header2")
font2:SetFont(GameTooltipText:GetFont(), 13)
font2:SetTextColor(1, .85, 0)
local font3 = CreateFont("Text")
font3:SetFont(GameTooltipText:GetFont(), 12)
local font4 = CreateFont("Numbers")
font4:SetFont(GameTooltipText:GetFont(), 10)

local classColors = {}
for class, color in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do
    classColors[class] = strformat( "%2x%2x%2x", color.r * 255, color.g * 255, color.b * 255 )
end

local bcm = LibStub("LibDataBroker-1.1"):NewDataObject("Broker Elements",{
    type = "data source",
    text = " ",
    label = "Broker Elements",
    icon = "Interface\\AddOns\\"..FOLDER_NAME.."\\icon",
    OnClick = function(self, button)
        if LTT:IsAcquired("BCMToolTip") then
            tooltip:Hide()
            LTT:Release(tooltip)
            tooltip = nil
        end
        if button == "RightButton" then
            fname = self:GetName()
            private.fname = fname
            ToggleDropDownMenu(1, nil, BCMDropDownMenu, fname, 0, 0)
        elseif
            button == "LeftButton" then
            if InterfaceOptionsFrame:IsShown() then
                HideUIPanel(InterfaceOptionsFrame)
            else
                InterfaceOptionsFrame_OpenToCategory(core.optFrames.core)
            end
        end
    end,
    OnEnter = function(self)
        if LTT:IsAcquired("BCMToolTip") then
            LTT:Release(tooltip)
            tooltip = nil
        end
        tooltip = LTT:Acquire("BCMToolTip", 3)
        tooltip:SetHeaderFont(Header1)
        tooltip:SetFont(Text)
        tooltip:SetScale(core.db.profile.scale)
        core:TooltipDraw()
        tooltip:SmartAnchorTo(self)
        tooltip:SetAutoHideDelay(.1, self)
        tooltip:EnableMouse()
        tooltip:SetScrollStep(core.db.profile.scroll - 25)
        tooltip:UpdateScrolling(core.db.profile.scroll)
        tooltip:Show()
    end,
} )

function core:TooltipDraw()
    if not private.scanPending then
        core:playerInfo()
    else
        local col= tooltip:GetColumnCount()
        tooltip:SetCell(tooltip:AddLine(), 1, "|cffffff00"..L["Guild Scan Pending"].."|r", col)
    end
end

function core:onTextUpdate()
    if not private.scanPending then
        tracktext = ""
        xp = private.bcitems
        for k,itemid in pairs (xp.id) do
            local text = core:textInfo(k, xp, itemid, item)
            tracktext = core:createText(tracktext, text, itemid)
        end
        xp = private.lkitems
        for k,itemid in pairs (xp.id) do
            local text = core:textInfo(k, xp, itemid, item)
            tracktext = core:createText(tracktext, text, itemid)
        end
        xp = private.cataitems
        for k,itemid in pairs (xp.id) do
            local text = core:textInfo(k, xp, itemid, item)
            tracktext = core:createText(tracktext, text, itemid)
        end
        xp = private.mopitems
        for k,itemid in pairs (xp.id) do
            local text = core:textInfo(k, xp, itemid, item)
            tracktext = core:createText(tracktext, text, itemid)
        end
        xp = private.woditems
        for k,itemid in pairs (xp.id) do
            local text = core:textInfo(k, xp, itemid, item)
            tracktext = core:createText(tracktext, text, itemid)
        end
        if tracktext == "" then
            tracktext = " "..("|cff00ff00"..L["Not Tracking"].."|r")
        end
    else
        tracktext = " "..("|cffffff00"..L["Guild Scan Pending"].."|r")
    end
   bcm.text = tracktext
end

function core:createText(tracktext, text, itemid)
    if text ~= nil then
        tracktext = tracktext.." "..text.." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv).." "
    end
    return tracktext
end

function core:textInfo(k, xp, itemid, item)
    local iconpath = GetItemIcon(itemid)
    local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
    local color
    if core.db.profile.textColor then
        color = xp.color[k]
    else
        color = "FFFFFF"
    end
    local name = xp.strings[k]
    local item
    local fixeditem = strsub(itemid, 1, -1)
    if core.db.profile.tracking[fixeditem] then
        if not core.db.profile.showShort then
            item = "|cff"..color..name.."|r"
        else
            local shortname = xp.short[k]
            item = "|cff"..color..shortname.."|r"
        end
        if core.db.profile.showIcons and not core.db.profile.iconsOnly then
            item = icon..item
        elseif core.db.profile.showIcons and core.db.profile.iconsOnly then
            item = icon
        else
            item = item
        end
    end
    return item
end

function core:playerInfo()
    local col, picol, pbcol, pgcol, hcol, cols = 0, 0, 0, 0, 3, 1
    local name = "|cff"..classColors[core.db.realm.chars[self.charName].class]..strsub(self.charName, 1, 20).."|r"
    tooltip:AddLine()
    tooltip:AddSeparator(2, 1, .85, 0, 1)
    tooltip:AddLine()
    tooltip:SetCell(3, 1, L["Item"], Header2)
    tooltip:SetColumnColor(2, 1, .85, 0, 1)
    tooltip:SetCell(3, 2, "", Header2)
    tooltip:SetCell(3, 3, L["Inv"], Header2)
    picol = tooltip:GetColumnCount()
    tooltip:AddSeparator(2, 1, .85, 0, 1)
    if  self.db.profile.showBank then
        tooltip:AddColumn()
        pbcol = tooltip:GetColumnCount()
        cols = cols + 1
        tooltip:SetCell(3, pbcol, L["Bank"], Header2)
    end
    if self.db.profile.showGuild then
        tooltip:AddColumn()
        pgcol = tooltip:GetColumnCount()
        cols = cols + 1
        tooltip:SetCell(3, pgcol, L["Guild"], Header2)
    end
    col = tooltip:GetColumnCount()
    tooltip:SetCell(1, hcol, name, Header1, col - 2)
    if core.db.profile.showAlts then core:altInfo() end
    if core.db.profile.modules.cmbc then
        xp = private.bcitems
        xpr, xpg, xpb, xpa = 0, 1, 0, .15
        core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols)
    end
    if core.db.profile.modules.cmlk then
        xp = private.lkitems
        xpr, xpg, xpb, xpa = 0, .1 ,1, .15
        core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols)
    end
    if core.db.profile.modules.cmcata then
        xp = private.cataitems
        xpr, xpg, xpb, xpa = .9, .2, 0, .15
        core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols)
    end
    if core.db.profile.modules.cmmop then
        xp = private.mopitems
        xpr, xpg, xpb, xpa = .2, .8, .9, .15
        core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols)
    end
    if core.db.profile.modules.cmwod then
        xp = private.woditems
        xpr, xpg, xpb, xpa = .95, .28, .18, .15
        core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols)
    end	
end

function core:altInfo()
    local itemcols = 1
    for char, _ in pairs(core.db.realm.chars) do
        if char ~= self.charName then
            local name = "|cff"..classColors[core.db.realm.chars[char].class]..strsub(char, 1, 20).."|r"
            tooltip:AddColumn()
            col = tooltip:GetColumnCount()
            tooltip:SetColumnColor(col, 1, .85, 0, 1)
            tooltip:SetCell(3, col, "", Header2)
            itemcols = itemcols + 2
            tooltip:AddColumn()
            aicol = tooltip:GetColumnCount()
            tooltip:SetCell(3, aicol, L["Inv"], Header2)
            if  self.db.profile.showBank then
                itemcols = itemcols + 1
                tooltip:AddColumn()
                abcol = tooltip:GetColumnCount()
                tooltip:SetCell(3, abcol, L["Bank"], Header2)
            end
            if self.db.profile.showGuild then
                itemcols = itemcols + 1
                tooltip:AddColumn()
                agcol = tooltip:GetColumnCount()
                tooltip:SetCell(3, agcol, L["Guild"], Header2)
            end
            ahcol = itemcols + 2
            tooltip:SetCell(1, ahcol, name, Header1, col - ahcol + 1)
        end
    end
    tooltip:AddColumn()
end

function core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols)
    cols = cols - 1
    for k, itemid in pairs(xp.id) do
        if core.db.profile.showZero or core:CharHasElement(itemid) then
            local line = tooltip:AddLine()
            if core.db.profile.lineColor then tooltip:SetLineColor(line, xpr, xpg, xpb, xpa) end
            tooltip:SetCell(line, 1, core:itemText(xp, k, itemid), Text)
            char = self.charName
            tooltip:SetCell(line, picol, core:InvCountString(itemid, char), Numbers)
            if  self.db.profile.showBank then
                tooltip:SetCell(line, pbcol, core:BankCountString(itemid, char), Numbers)
            end
            if  self.db.profile.showGuild then
                tooltip:SetCell(line, pgcol, core:GuildCountString(itemid, char), Numbers)
            end
            if core.db.profile.showAlts then
                local x, aicol, abcol, agcol = 0, 0, 0, 0
                for char, _ in pairs(core.db.realm.chars) do
                    if char ~= self.charName then
                        x = x + 1
                        aicol = picol + 2 * x + cols * x
                        tooltip:SetCell(line, aicol, core:InvCountString(itemid, char), Numbers)
                        if  self.db.profile.showBank then
                            abcol = pbcol + 2 * x + cols * x
                            tooltip:SetCell(line, abcol, core:BankCountString(itemid, char), Numbers)
                        end
                        if  self.db.profile.showGuild then
                            agcol = pgcol + 2 * x + cols * x
                            tooltip:SetCell(line, agcol, core:GuildCountString(itemid, char), Numbers)
                        end
                    end
                end
            end
        end
    end
end

function core:itemText(xp, k, itemid)
    local iconpath = GetItemIcon(itemid)
    local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
    local color
    if core.db.profile.textColor then
        color = xp.color[k]
    else
        color = "FFFFFF"
    end
    local name = xp.strings[k]
    local item
    if not core.db.profile.tooltipShowShort then
        item = "|cff"..color..name.."|r"
    else
        local shortname = xp.short[k]
        item = "|cff"..color..shortname.."|r"
    end
    if core.db.profile.tooltipIcons and not core.db.profile.tooltipIconsOnly then
        item = icon .. " " .. item
    elseif core.db.profile.tooltipIcons and core.db.profile.tooltipIconsOnly then
        item = icon
    else
        item = item
    end
    return item
end